/*
 * analog.c
 *
 * Created: 2014-11-21 07:20:06
 *  Author: dsosnowski
 */ 

#define F_CPU 8000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>
#include "defs.h"
#include "analog.h"
#include "serial.h"
#include "settings.h"

struct {
	char comp_vin;
	char comp_vout;
	char comp_curr10;
	char comp_curr100;
	uint16_t vin;
	uint16_t vout;
	uint32_t curr;
	uint16_t counter;
	char range;
	char status;
} acc;

struct {
	uint16_t vref;
	uint16_t resist;
	uint16_t vin;
	uint16_t vout;
	uint16_t curr;
} analogval;

struct {
	char status;
	char restart_bit;
	uint32_t count;
	uint32_t power;

	uint32_t cap;
	uint32_t capact;
	
	uint32_t energy;
	uint32_t energyact;
	uint16_t energy_bit;
} poweracc;

//----------------------------------
void AnalogInit(void)
{
	ADCSRA = (1<<ADEN | 1<<ADIE | 1<<ADPS2 | 1<<ADPS1 | 1<<ADPS0);
	//ADCSRA = (1<<ADEN | 1<<ADPS2 | 1<<ADPS1 | 1<<ADPS0);
	ADMUX = 0;
	ADCSRA |= (1<<ADSC);
	analogval.vref = ANALOG_DEFAULT_VREF;
	analogval.resist = ANALOG_DEFAULT_R;
}

//----------------------------------
uint16_t AnalogGet(char channel)
{
	while (ADCSRA & (1<<ADSC));
	ADMUX = channel;
	//ADCSRA |= (1<<ADSC);
		
	ADCSRA |= (1<<ADIF);
	MCUCR = (1<<SE | 1<<SM0);
	sleep_cpu();	
	MCUCR = 0;
	
	
	while (ADCSRA & (1<<ADSC));
	return ADC;
}


//----------------------------------
void AnalogCalib(void)
{
	uint16_t temp;
	char i;
	char j;
	char *pt;
	
	i = 0;
	pt = &acc.comp_vin;

	while (i < 4)
	{
		j = 0;
		temp = 0;	
		while (j < 20)
		{
			if (i == 0)	temp += AnalogGet(VINPUT_PIN);
			if (i == 1)	temp += AnalogGet(VOUTPUT_PIN);
			if (i == 2)	temp += AnalogGet(CURR10_PIN);
			if (i == 3)	temp += AnalogGet(CURR100_PIN);
			j++;
		}
		temp /= j;
		*pt = (char)temp;		
		pt++;
		i++;
	}
}


//----------------------------------
char AnalogProc(void)
{
	uint16_t temp;
	uint16_t temp2;
	//static char range;
	
	if (acc.status == ADC_STATUS_OK) 
	{		
		temp = AnalogGet(VINPUT_PIN);
		if (temp > acc.comp_vin) temp -= acc.comp_vin;
		else temp = 0;
		acc.vin += temp;
		//acc.vin += 1000;

		temp = AnalogGet(VOUTPUT_PIN);
		if (temp > acc.comp_vout) temp -= acc.comp_vout;
		else temp = 0;
		acc.vout += temp;
		//acc.vout += 500;

		temp = AnalogGet(CURR10_PIN);
		temp2 = AnalogGet(CURR100_PIN);

		if (temp > acc.comp_curr10) 
			temp -= acc.comp_curr10;
		else temp = 0;
		
		if (temp2 > acc.comp_curr100) 
			temp2 -= acc.comp_curr100;
		else temp2 = 0;

		if (temp <= ADC_LSWITCHRANGE_VAL) acc.range = 1;
		if (temp >= ADC_HSWITCHRANGE_VAL) acc.range = 0;

		if (acc.range == 1)
			temp = temp2;	
		else
			temp *= ANALOG_CRANGE0_MULTI;

		acc.curr += temp;	
		//acc.curr += 2000;

		acc.counter++;
		if (acc.counter >= ADC_COUNTER_MAX) acc.status = ADC_READY;
	}
	//else
	//	acc.status = ADC_READY;
		
	return acc.status;
}

//----------------------------------
void AnalogProcRestart(void)
{
	acc.vin = 0;
	acc.vout = 0;
	acc.counter = 0;
	acc.curr = 0;
	acc.status = ADC_STATUS_OK;
}

//----------------------------------
ISR(ADC_vect)
{	
}

//----------------------------------
void AnalogValuesCalc(void)
{
	float fval;

	fval = acc.vin;
	fval = (fval / acc.counter);
	fval = (fval * analogval.vref);
	fval = (fval / ANALOG_ADC_RES);
	fval = (fval * ANALOG_VBLOCK_MULTI);
	analogval.vin = (uint16_t)fval;
	
	fval = acc.vout;
	fval = (fval / acc.counter);
	fval = (fval * analogval.vref);
	fval = (fval / ANALOG_ADC_RES);
	fval = (fval * ANALOG_VBLOCK_MULTI);
	analogval.vout = (uint16_t)fval;
	
	fval = acc.curr;
	fval = (fval / acc.counter);
	fval = (fval * analogval.vref);
	fval = (fval / ANALOG_ADC_RES);
	fval = (fval * ANALOG_CBLOCK_MULTI);
	fval = (fval / analogval.resist);
	analogval.curr = (uint16_t)fval;

	//vout correction
	fval = (fval * analogval.resist);
	fval = (fval / 1000);
	if (analogval.vout > (uint16_t)fval) 
		analogval.vout -= (uint16_t)fval;
	else
		analogval.vout = 0;
}

//----------------------------------
void PowerCalc(void)
{
	uint32_t powercalc;
	
	if (poweracc.count < POWER_COUNT_MAX)
	{
		//time
		//if (poweracc.restart_bit == 1) poweracc.count++;
		//poweracc.restart_bit = 1;
		poweracc.count++;
		
		//cap
		poweracc.cap += analogval.curr;
		if (poweracc.cap > POWER_ACC_CAP_MAX) 
			poweracc.cap = POWER_ACC_CAP_MAX;
		poweracc.capact	= (poweracc.cap / 3600);
		
		//pow
		powercalc = analogval.curr;
		if (SettVchannel(0) <= 1)
			powercalc = (powercalc * analogval.vin);
		else
			powercalc = (powercalc * analogval.vout);
		poweracc.power = (powercalc / 1000);
		
		//energy
		powercalc += poweracc.energy_bit;
		poweracc.energy_bit = (powercalc % 1000);
		poweracc.energy += (powercalc / 1000);
		if (poweracc.energy > POWER_ACC_POWER_MAX)
			poweracc.energy = POWER_ACC_POWER_MAX;
		
		poweracc.energyact = (poweracc.energy / 3600);	
	}
}

//----------------------------------
void PowerRestart(void)
{
	analogval.curr = 0;
	analogval.vin = 0;
	analogval.vout = 0;

	poweracc.restart_bit = 0;
	poweracc.count = 0;
	
	poweracc.cap = 0;
	poweracc.capact = 0;
	
	poweracc.energy = 0;
	poweracc.energyact = 0;
	poweracc.energy_bit = 0;	
}

//----------------------------------
char PowerProcess(char action)
{
	if (action == POWER_PROCESS_START)				//start
	{
		if (poweracc.status <= POWER_PROCESS_IDLE)	//if idle
		{
			poweracc.status = POWER_PROCESS_WORK;	//start
		}
	}
	if (action == POWER_PROCESS_STOP)				//stop
	{
		if (poweracc.status == POWER_PROCESS_READY)	//if stop
		{
			PowerRestart();
			poweracc.status = POWER_PROCESS_IDLE;	//reset & idle
		}
		if (poweracc.status == POWER_PROCESS_WORK)	//if working
		{
			poweracc.status = POWER_PROCESS_READY;	//stop
		}
	}
	if ((poweracc.status == 0) || (poweracc.status > 3)) 
		poweracc.status = POWER_PROCESS_IDLE;
	
	return poweracc.status;
}

//----------------------------------
char PowerAutoStop(char action)
{
	char mode;
	static char accept;
	
	if (action) accept = 0;
	
	mode = SettStopMode(0);
	if (mode > 1)
	{
		if (mode == 2)
		{
			if (analogval.curr <= SettStopParam(2)) 
				accept++;
			else
				accept = 0;
		}
		if (mode == 3)
		{
			if (SettVchannel(0) <= 1)
			{
				if (analogval.vin <= SettStopParam(3)) 
					accept++;
				else
					accept = 0;
			}
			else
			{
				if (analogval.vout <= SettStopParam(3))
					accept++;
				else
					accept = 0;
			}
		}
		if (mode == 4)
		{
			if (SettVchannel(0) <= 1)
			{
				if (analogval.vin >= SettStopParam(4))
					accept++;
				else
					accept = 0;
			}
			else
			{
				if (analogval.vout >= SettStopParam(4))
					accept++;
				else
					accept = 0;
			}
		}
		if (mode == 5)
		{
			if (poweracc.count >= (uint32_t)(SettStopParam(5) * 60)) 
				accept = AUTOSTOP_ACCEPT_CYCLE;
			else
				accept = 0;

		}
		if (mode == 6)
		{
			if (poweracc.capact >= (uint32_t)(SettStopParam(6) * 100))
				accept = AUTOSTOP_ACCEPT_CYCLE;
			else
				accept = 0;

		}
		if (accept > AUTOSTOP_ACCEPT_CYCLE) accept = AUTOSTOP_ACCEPT_CYCLE;	
	}
	else
		accept = 0;
		
	return accept;

}