/*
 * serial.c
 *
 * Created: 2015-02-27 12:38:35
 *  Author: dsosnowski
 */ 



#define F_CPU 12000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "serial.h"
#include "defs.h"

#define CORE_CLOCK		8000000
#define BAUD_RATE		19200

#define SUART_PORT		UI_PORT
#define SUART_PIN		UI_PIN
#define SUART_DDR		UI_DDR

//-------------------------------------------
struct {
	char wbuff[SERIAL_WBUFFER_SIZE];
	uint16_t wcounter;
	volatile char *wpt;
	char rbuff[SERIAL_RBUFFER_SIZE];
	uint16_t rcounter;
	char command;
	char param;
	volatile char status;
}serial;

//-------------------------------------------
void SerialInit(void)
{
	SUART_PORT |= (1<<TXD_PIN | 1<<RXD_PIN);	//config ports
	SUART_DDR |= (1<<TXD_PIN);
	
	//config USART
	UCSRA = 0;
	UCSRB = (1<<RXEN | 1<<TXEN | 1<<TXCIE | 1<<RXCIE);		//RX
	UCSRC = (1<<UCSZ1 | 1<<UCSZ0);				//8 data, 1 stop, parity none
	UBRRH = 0;
	UBRRL = 25;									//UBRRL = 25 - 19.2k
	
	SerialBufferClear();
}

//-------------------------------------------
char *SerialGetWbuff(uint16_t i)
{
	return &serial.wbuff[i];
}

//-------------------------------------------
void SerialSend(char *data)
{
	while (*data > 0)
	{
		while(!(UCSRA & 1<<UDRE)){}
		UDR = *data;
		data++;
	}
}


//-------------------------------------------
void SerialBufferAdd(char *data)
{
	while (*data > 0)
	{
		if (serial.wcounter < (SERIAL_WBUFFER_SIZE - 1))
		{
			serial.wbuff[serial.wcounter] = *data;
			serial.wcounter++;			
		}
		data++;
	}
	serial.wbuff[serial.wcounter] = 0;
}

//-------------------------------------------
void SerialBufferSend(void)
{
	serial.wpt = &serial.wbuff[0];
	if (*serial.wpt > 0)
	{
		while(!(UCSRA & 1<<UDRE)){};
		UDR = *serial.wpt;	
	}
}

//-------------------------------------------
void SerialBufferClear(void)
{
	serial.wcounter = 0;
	serial.wbuff[0] = 0;
	serial.wpt = &serial.wbuff[0];	
}

//----------------------------------------------------
ISR(USART_TXC_vect)
{
	serial.wpt++;
	if (*serial.wpt > 0) 
		UDR = *serial.wpt;
}


//----------------------------------------------------
ISR(USART_RXC_vect)
{
	char temp = UDR;						//get byte from data register
	
	if (temp == COM_PREFIX)
	serial.rcounter = 0;
	
	if (serial.rcounter < SERIAL_RBUFFER_SIZE)
	{
		serial.rbuff[serial.rcounter] = temp;
		serial.rcounter++;
		serial.rbuff[serial.rcounter] = 0;
	}
	
	if (temp == COM_POSTFIX)
	serial.status = SERIAL_READY;
}

//-----------------------------------
void SerialVal(uint16_t value)
{
	uint16_t temp = 10000;
	char digit;
	char blank = 1;

	SerialChar('#');
	while (temp)
	{
		digit = value / temp;
		value = value % temp;
		
		if (temp == 1) blank = 0;
		if (digit) blank = 0;
		
		if(!(blank)) SerialChar(digit + '0');
		else SerialChar(' ');

		temp = temp / 10;
	}
	SerialChar('#');
}


//-------------------------------------
void SerialChar(char data)
{
	while(!(UCSRA & 1<<UDRE));
	UDR = data;
}