/*
 * settings.c
 *
 * Created: 2015-02-27 12:41:40
 *  Author: dsosnowski
 */ 
#define F_CPU 8000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>
#include "defs.h"
#include "ui.h"
#include "lcd.h"
#include "settings.h"
#include "analog.h"

struct {
	char start;		
	char stop;		//1-manual, 2-cmin, 3-vmin, 4-vmax, 5-tmax, 6-amax, 
	char vch;		//1-vin , 2-vout
	char usb;		//1-1s period...
	uint16_t param[7];	//stop param
} sett;

extern struct {
	char comp_vin;
	char comp_vout;
	char comp_curr10;
	char comp_curr100;
	uint16_t vin;
	uint16_t vout;
	uint32_t curr;
	uint16_t counter;
	char range;
	char status;
} acc;

/*//---------------------------------
char SettStartMode(char newval)
{
	if (newval > 0) sett.start = newval;
	if ((sett.start == 0) || (sett.start > 2)) sett.start = 1; 
	return sett.start;
}*/

//---------------------------------
char SettStopMode(char press)
{
	if ((press == SW_PLUS) && (sett.stop < 6)) sett.stop++;
	if ((press == SW_MINUS) && (sett.stop > 1)) sett.stop--;
	if ((sett.stop == 0) || (sett.stop > 6)) sett.stop = 1;
	return sett.stop;
}

//---------------------------------
uint16_t SettStopParam(uint16_t index)
{	
	if (sett.stop == 0) sett.param[0] = 0;
	if (sett.stop == 1) sett.param[1] = 0;
	if ((sett.stop == 2) && (sett.param[2] >  ANALOG_CURR_MAX)) sett.param[2] = 0;
	if ((sett.stop == 3) && (sett.param[3] >  ANALOG_VOLTAGE_MAX)) sett.param[3] = 0;
	if ((sett.stop == 4) && (sett.param[4] >  ANALOG_VOLTAGE_MAX)) sett.param[4] =0;
	if ((sett.stop == 5) && (sett.param[5] >  6000)) sett.param[5] = 0;
	if ((sett.stop == 6) && (sett.param[6] >  1000)) sett.param[6] = 0;
	return sett.param[index];
}

//---------------------------------
char SettVchannel(char press)
{
	if ((press == SW_PLUS) && (sett.vch < 2)) sett.vch++;
	if ((press == SW_MINUS) && (sett.vch > 1)) sett.vch--;
	if ((sett.vch == 0) || (sett.vch > 2)) sett.vch = 1;
	return sett.vch;
}

//---------------------------------
char SettUsbperiod(char press)
{
	if ((press == SW_PLUS) && (sett.usb < 250)) sett.usb++;
	if ((press == SW_MINUS) && (sett.usb > 1)) sett.usb--;
	if (sett.usb > 250) sett.usb = 1;
	return sett.usb;
}

//---------------------------------
void SettAllRestore(void)
{
	SettRestore(&acc.comp_vin, 6, 4);
	if (acc.comp_vin == 255) acc.comp_vin = 0;
	if (acc.comp_vout == 255) acc.comp_vout = 0;
	if (acc.comp_curr10 == 255) acc.comp_curr10 = 0;
	if (acc.comp_curr100 == 255) acc.comp_curr100 = 0;
		
	SettRestore(&sett.start, 10, 18);
	//SettStartMode(0);
	SettStopMode(0);
	SettStopParam(0);
	SettVchannel(0);
	SettUsbperiod(0);
}

//---------------------------------
void SettAllStore(void)
{
	SettStore(&acc.comp_vin, 6, 4);
	SettStore(&sett.start, 10, 18);
}

//---------------------------------
void SettStore(char *pt, char adr, char len)
{
	while (len)
	{
		EEwrite(adr, *pt);
		adr++;
		pt++;
		len--;
	}
}

//---------------------------------
void SettRestore(char *pt, char adr, char len)
{
	while (len)
	{
		*pt = EEread(adr);
		adr++;
		pt++;
		len--;
	}
}
/*//---------------------------------
void EEwrite16(uint16_t address, int16_t data16)
{
	EEwrite(address, (char)(data16 & 0xFF));
	data16 >>= 8;
	EEwrite(address + 1, (char)(data16 & 0xFF));
}

//---------------------------------
int16_t EEread16(uint16_t address)
{
	int16_t data16 = 0;
	data16 += EEread(address + 1);
	data16 <<= 8;
	data16 += EEread(address);
	return data16;
}*/

//---------------------------------
void EEwrite(uint16_t address, char data)
{
	while(EECR & (1<<EEWE)); 		//Wait for completion of previous write
	EECR = 0;						//Set Programming mode

	EEAR = address;					//Set up address and data registers
	EEDR = data;

	EECR |= (1<<EEMWE);				//Write logical one to EEMPE
	EECR |= (1<<EEWE);				//Start eeprom write by setting EEPE
}

//---------------------------------
char EEread(uint16_t address)
{
	while(EECR & (1<<EEWE));		//Wait for completion of previous write
	
	EEAR = address;					//Set up address register
	EECR |= (1<<EERE);				//Start eeprom read by writing EERE
	
	return EEDR;					//Return data from data register
}


//---------------------------------
void SettingsValChange(char level, char press)
{
	if (level == 2)
	{
			if (sett.stop == 2)
			{
				if (press == SW_PLUS) sett.param[2] += 10;
				if (press == SW_MINUS) sett.param[2] -= 10;
				if (sett.param[2] > ANALOG_CURR_MAX)
					sett.param[2] = 0; 
			}
			if ((sett.stop == 3) || (sett.stop == 4))
			{
				if (press == SW_PLUS) sett.param[sett.stop] += 100;
				if (press == SW_MINUS) sett.param[sett.stop] -= 100;
				if (sett.param[sett.stop] > ANALOG_VOLTAGE_MAX)
				sett.param[sett.stop] = 0;
			}
			if (sett.stop == 5)
			{
				if (press == SW_PLUS) sett.param[5] += 10;
				if (press == SW_MINUS) sett.param[5] -= 10;
				if (sett.param[5] > 6000)
				sett.param[5] = 0;
			}
			if (sett.stop == 6)
			{
				if (press == SW_PLUS) 
				{
					sett.param[6] += 1;					
				}
				if (press == SW_MINUS) 
				{
					sett.param[6] -= 1;
				}
				
				if (sett.param[6] > 6000)
				sett.param[6] = 0;
			}
	}
	if (level == 4)
		SettUsbperiod(press);
}