/*
 * ui.c
 *
 * Created: 2014-09-17 13:57:56
 *  Author: dsosnowski
 */ 
#define F_CPU 8000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>
#include "defs.h"
#include "ui.h"
#include "lcd.h"
#include "serial.h"
#include "analog.h"
#include "settings.h"

struct {
	char press;
	char level;
	char param;
	char enter;
	char refresh;
} ui;

extern struct {
	char comp_vin;
	char comp_vout;
	char comp_curr10;
	char comp_curr100;
	uint16_t vin;
	uint16_t vout;
	uint32_t curr;
	uint16_t counter;
	char range;
	char status;
} acc;

extern struct {
	uint16_t vref;
	uint16_t resist;
	uint16_t vin;
	uint16_t vout;
	uint16_t curr;
} analogval;

struct {
	char conv[12];
} contents ;

//const char up[8] = {0x04, 0x0E, 0x1F, 0x04, 0x04, 0x04, 0x04, 0x00};
//const char down[8] = {0x04, 0x04, 0x04, 0x04, 0x1F, 0x0E, 0x04, 0x00};


//----------------------------------------------
void UiInit(void)
{
	UI_PORT |= (1<<SW1_PIN | 1<<SW2_PIN | 1<<SW3_PIN | 1<<BKL_PIN);
	
	//LCDSetSpecChar(0, up);
	//LCDSetSpecChar(1, down);
/*	LCDSetSpecChar(0, fall);
	LCDSetSpecChar(1, rise);
	LCDSetSpecChar(3, nn); 
	LCDSetSpecChar(4, uu);
	LCDSetSpecChar(5, ss);
	LCDSetSpecChar(6, zz);
*/
}

//----------------------------------------------
char UiScan(void)
{
	char pin;
	char key;
	static char prev;
	static char hold;
	
	pin = UI_PIN;
	key = 0;
	
	//any state
	if ((pin & (1<<SW1_PIN | 1<<SW2_PIN | 1<<SW3_PIN)) == (1<<SW1_PIN | 1<<SW2_PIN | 1<<SW3_PIN))
	{
		//sw_free
		if ((hold > 0) && (hold < SWITCH_THRESHOLD)) key = prev;
		prev = 0;			
		hold = 0;
	}
	else
	{
		//if pressed
		if ((pin & (1<<SW1_PIN)) == 0)	key = 1;
		if ((pin & (1<<SW2_PIN)) == 0)	key = 2;
		if ((pin & (1<<SW3_PIN)) == 0)	key = 3;

		if (hold < 200) hold++;			//counting hold time
		if (key != prev) hold = 0;		//new key - reset counters
		prev = key;
		key = 0;

		if (hold == SWITCH_REPEAT) //after threshold - repeat key
		{
			key =  (prev + SWITCH_HOLD);
			if ((prev == SW_PLUS) || (prev == SW_MINUS))
				hold = SWITCH_THRESHOLD;
		}
	}
	
	ui.press = key;
	return key;
}

//----------------------------------------------
//Main, Auto, Stop, Vch, USB, Calib,
void UiNavi(char press)
{
		switch (press)
		{
			case SW_MENU:
				ui.enter = 0;
				ui.level++;
				if (ui.level > 4) ui.level = 0;
			break;
			case SW_PLUS:
				if (ui.level == 0)	PowerProcess(POWER_PROCESS_START);
				else
				{
					if (ui.level == 1)	SettStopMode(press);
					if (ui.level == 2)	SettingsValChange(ui.level, press);
					if (ui.level == 3)	SettVchannel(press);
					if (ui.level == 4)	SettUsbperiod(press);
					if (ui.level == 5)
					{
						if (ui.enter < 2) ui.enter++;
						else ui.enter = 0;
						
						if (ui.enter == 2)
						SettStore(&acc.comp_vin, 6, 4);
					}
				}
			break;
			case SW_MINUS:
				if (ui.level == 0)	PowerProcess(POWER_PROCESS_START);
				else
				{
					if (ui.level == 1)	SettStopMode(press);
					if (ui.level == 2)	SettingsValChange(ui.level, press);
					if (ui.level == 3)	SettVchannel(press);
					if (ui.level == 4)	SettUsbperiod(press);
					if (ui.level == 5)
					{
						if (ui.enter > 0) ui.enter--;
						
						if (ui.enter == 2)
						SettStore(&acc.comp_vin, 6, 4);
					}
				}
			break;
			case SW_PLUSH:
				if (ui.level > 0)
				{
					SettingsValChange(ui.level, SW_PLUS);
				}
			break;
			case SW_MINUSH:
				if (ui.level > 0)
				{
					SettingsValChange(ui.level, SW_MINUS);
				}
			break;			
			case SW_MENUH:
				if (ui.level == 0)	PowerProcess(POWER_PROCESS_STOP);			
			break;
		}	
}

//----------------------------------------------
void UiMenu(char level)
{
	char temp;
	
	if (level == 0)			
	{
		LCDLocate(0,0);
		LCDFrame(SerialGetWbuff(48), 7);	//A)
		LCDText("  ");
		LCDFrame(SerialGetWbuff(12), 7);	//Ah)
		LCDLocate(1,0);
		LCDFrame(SerialGetWbuff(39), 7);	//V)
		LCDText(" ");
		LCDFrame(SerialGetWbuff(2), 8);		//T)
	}

	if (level == 1)
	{
		LCDLocate(0,0);
		LCDText("   Stop mode    ");
		LCDLocate(1,0);
		temp = SettStopMode(0);
		if (temp <= 1) 
			LCDText("         Manual ");
		if (temp == 2)
			LCDText("     Auto. Cmin ");
		if (temp == 3)
			LCDText("     Auto. Vmin ");
		if (temp == 4)
			LCDText("     Auto. Vmax ");
		if (temp == 5)
			LCDText("     Auto. Time ");
		if (temp == 6)
			LCDText("     Auto. Cap ");
	}
	
	if (level == 2)
	{
		LCDLocate(0,0);
		temp = SettStopMode(0);
		if (temp <= 1)
		{
			ui.enter = 0;
			LCDText(" Auto stop dis  ");
			LCDLocate(1,0);
			LCDText("                ");
		}
		else
		{
			LCDText(" Auto. param.   ")	;
			LCDLocate(1,0);
			
			if (temp == 2)
			{
				LCDText(" Cmin [A]  ");
				LCDFrame(Ui4Val2str(SettStopParam(2) / 10, 3), 5);
			}
			if (temp == 3)
			{
				LCDText(" Vmin [V]  ");
				LCDFrame(Ui4Val2str(SettStopParam(3) / 10, 3), 5);
			}
			if (temp == 4)
			{
				LCDText(" Vmax [V]  ");
				LCDFrame(Ui4Val2str(SettStopParam(4) / 10, 3), 5);
			}
			if (temp == 5)
			{
				LCDText(" Time   ");
				LCDFrame(UiTime2str((uint32_t)SettStopParam(5) * 60), 8);
			}
			if (temp == 6)
			{
				LCDText(" Cap [Ah]  ");
				LCDFrame(Ui4Val2str(SettStopParam(6), 2), 5);
			}
			if (ui.enter)
			{
				LCDLocate(1,0);
				LCDData('>');
			}
		}
	}
	
	if (level == 3)
	{
		LCDLocate(0,0);
		LCDText("   V channel    ");
		LCDLocate(1,0);
		if (SettVchannel(0) <= 1) 
			LCDText("          Input ");
		//if (SettVchannel(0) == 2) 
		else
			LCDText("         Output ");
	}
	
	if (level == 4)
	{
		LCDLocate(0,0);
		LCDText("   USB period   ");
		LCDLocate(1,0);
		LCDText("        ");
		LCDFrame(UiTime2str(SettUsbperiod(0)), 8);
		if (ui.enter)
		{
			LCDLocate(1,0);
			LCDData('>');
		}	
	}
	
	if (level == 5)
	{
		if (ui.enter == 0)
		{
			LCDLocate(0,0);
			LCDText("  Calibration   ");		
			LCDLocate(1,0);
			LCDText("                ");			
		}

		if (ui.enter == 1)
		{
			AnalogCalib();
			LCDLocate(0,0);
			LCDText("  Zero comp.    ");
			LCDLocate(1,0);
			LCDFrame(Ui4Val2str(acc.comp_curr10, 0), 4);
			LCDFrame(Ui4Val2str(acc.comp_curr100, 0), 4);
			LCDFrame(Ui4Val2str(acc.comp_vin, 0), 4);
			LCDFrame(Ui4Val2str(acc.comp_vout, 0), 4);						
		}

		if (ui.enter == 2)
		{
			LCDLocate(0,0);
			LCDFrame(Ui4Val2str(analogval.curr / 10, 3), 5);			//A)
			LCDText("A   Range ");
			LCDData(acc.range + '0');
			
			LCDLocate(1,0);
			LCDFrame(Ui4Val2str(analogval.vin / 10, 3), 5);			//A)
			LCDText("Vi  ");
			LCDFrame(Ui4Val2str(analogval.vout / 10, 3), 5);			//A)
			LCDText("Vo");
		}
	}
}

//----------------------------------------------
/*void UiSerial2Disp(char loc, char param)
{
	char *pt;
	
	if (loc == 0) LCDLocate(0,0);
	if (loc == 1) LCDLocate(1,0);
	if (loc == 2) LCDLocate(0,8);
	if (loc == 3) LCDLocate(1,8);
	
	if (loc >= 2)
	{
		 if (param != 0) LCDText(" ");
	}
	
	if (param == 0) pt = SerialGetWbuff(2);		//time
	if (param == 1) pt = SerialGetWbuff(12);	//Ah
	if (param == 2) pt = SerialGetWbuff(21);	//Wh
	if (param == 3) pt = SerialGetWbuff(30);	//W
	if (param == 4) pt = SerialGetWbuff(39);	//V
	if (param == 5) pt = SerialGetWbuff(48);	//A
	
	while((*pt > 0) && (*pt != ';'))
	{
		LCDData(*pt);
		pt++;
	}
	
	if (loc < 2)
	{
		if (param != 0) LCDText(" ");
	}
	
}*/

//----------------------------------------------
char* Ui4Val2str(uint16_t val, char dot)
{
	char *pt;
	pt = &contents.conv[0];
	
	char blink = 0;
	char d;
	char i = 4;
	uint16_t div = 10000;
	if(val > 9999) val = 9999;
	
	while (i)
	{
		val = (val % div);
		div = (div / 10);
		
		d = (val / div);
		if (d) blink = 1;
		if (i == 1) blink = 1;
		if ((dot == i) && (dot > 0)) blink = 1;
		

		if (blink) *pt = (d + '0');
		else *pt = (' ');
		pt++;
		
		if ((dot == i) && (dot > 0))		
		{
			*pt = '.';
			pt++;
		}
				
		i--;
	}
	*pt = 0;
	return &contents.conv[0];
}

//----------------------------------------------
char* UiTime2str(uint32_t timeval)
{
	char *pt;
	pt = &contents.conv[0];
	char d;
	
	if (timeval > 359999) timeval = 359999;
	d = (timeval / 36000);	
	if (d == 0) *pt = ' ';
	else *pt = (d + '0');
	pt++;
	
	timeval %= 36000;	
	d = (timeval / 3600);
	*pt = d + '0'; pt++; 
	*pt = ':'; pt++;
	
	timeval %= 3600;
	d = (timeval / 600);
	*pt = d + '0'; pt++;

	timeval %= 600;
	d = (timeval / 60);
	*pt = d + '0'; pt++; 
	*pt = ':'; pt++;

	timeval %= 60;
	d = (timeval / 10);
	*pt = d + '0'; pt++;
	
	d = (timeval % 10);
	*pt = d + '0'; pt++;
	
	*pt = 0;
	return &contents.conv[0];
}